@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2012-2013)
+#   Copyright (C) Dr. Heinz-Josef Claes (2012-2014)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -98,8 +98,7 @@ require 'checkObjPar.pl';
 require 'prLog.pl';
 require 'version.pl';
 require 'fileDir.pl';
-
-
+require 'dateTools.pl';
 
 my $tmpdir = '/tmp';              # default value
 $tmpdir = $ENV{'TMPDIR'} if defined $ENV{'TMPDIR'};
@@ -116,8 +115,10 @@ linkToDirs.pl - hard links files in directories with others
 
 	linkToDirs.pl [--linkWith copyBackupDir] [--linkWith ...]
 		      --targetDir targetForSourceDir
-		      [--progressReport number] [--dontLinkSymlinks]
+		      [--progressReport number[,timeframe]] 
+		       [--printDepth] [--dontLinkSymlinks]
 		      [--ignoreErrors] [--saveRAM] [-T tmpdir]
+		      [--createSparseFiles [--blockSize]]
 		      sourceDir ...
 
 =head1 DESCRIPTION
@@ -155,6 +156,14 @@ linked always in targetDir.
 =item B<--progressReport>, B<-P>
 
     print progress report after each 'number' files
+    additional you may add a time frame after which a message is
+    printed
+    if you want to print a report each 1000 files and after
+    one minute and 10 seconds, use: -P 1000,1m10s
+
+=item B<--printDepth>, B<-D>
+
+    print depth of actual read directory during backup
 
 =item B<--dontLinkSymlinks>
 
@@ -172,6 +181,19 @@ linked always in targetDir.
 =item B<--tmpdir>, B<-T>
 
     directory for temporary files, default is </tmp>
+
+=item B<--createSparseFiles>, B<-s>
+
+    if a file is indicated as a sparse file and that file has to be
+    copied, then external program 'cp' is called to copy that file
+    (gnucp / linux does some inspection about sparse files, if your
+    OS related version of cp does not support this functionality,
+    then this option will not work for you)
+
+=item B<--blockSize>
+
+    block size used to check if a file is / may be a sparse file
+    default is 512 bytes (which should be fine for most file systems)
     
 =item B<sourceDir>
 
@@ -182,17 +204,15 @@ linked always in targetDir.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2012-2013 by Heinz-Josef Claes (see README).
+Copyright (c) 2012-2014 by Heinz-Josef Claes (see README).
 Published under the GNU General Public License v3 or any later version
 
 =cut
 
 
-my $Help = join('', grep(!/^\s*$/, `pod2text "$0"`));
-$Help = "cannot find pod2text, see documentation for details\n"
-    unless $Help;
+my $Help = &::getPod2Text($0);
 
-&printVersion(\@ARGV, '-V');
+&printVersion(\@ARGV, '-V', '--version');
 
 my $CheckPar =
     CheckParam->new('-allowLists' => 'yes',
@@ -209,8 +229,13 @@ my $CheckPar =
 				Option->new('-name' => 'progressReport',
 					    '-cl_option' => '--progressReport',
 					    '-cl_alias' => '-P',
-					    '-default' => 0,
-					    '-pattern' => '\A\d+\Z'),
+					    '-cf_key' => 'progressReport',
+					    '-default' => 0),
+				Option->new('-name' => 'printDepth',
+					    '-cl_option' => '--printDepth',
+					    '-cl_alias' => '-D',
+					    '-cf_key' => 'printDepth',
+					    '-cf_noOptSet' => ['yes', 'no']),
 				Option->new('-name' => 'dontLinkSymlinks',
 					    '-cl_option' => '--dontLinkSymlinks'),
 				Option->new('-name' => 'ignoreErrors',
@@ -218,10 +243,17 @@ my $CheckPar =
 					    '-cl_alias' => '--ignoreErrors'),
 				Option->new('-name' => 'saveRAM',
 					    '-cl_option' => '--saveRAM'),
+				Option->new('-name' => 'createSparseFiles',
+					    '-cl_option' => '--createSparseFiles',
+					    '-cl_alias' => '-s'),
+				Option->new('-name' => 'blockSize',
+					    '-cl_option' => '--blockSize',
+					    '-default' => 512,
+					    '-only_if' => '[createSparseFiles]',
+					    '-pattern' => '\A\d+\Z'),
 				Option->new('-name' => 'tmpdir',
 					    '-cl_option' => '-T',
 					    '-cl_alias' => '--tmpdir',
-					    '-cf_key' => 'tmpDir',
 					    '-default' => $tmpdir)
 				]);
 
@@ -233,13 +265,16 @@ my $linkWith = $CheckPar->getOptWithPar('linkWith');
 @$linkWith = () unless $linkWith;
 my $targetDir = $CheckPar->getOptWithPar('targetDir');
 my $progressReport = $CheckPar->getOptWithPar('progressReport');
+my $printDepth = $CheckPar->getOptWithoutPar('printDepth');
+$printDepth = $printDepth ? 'yes' : 'no';
 my $dontLinkSymlinks = $CheckPar->getOptWithoutPar('dontLinkSymlinks');
 my $ignoreErrors = $CheckPar->getOptWithoutPar('ignoreErrors');
 my (@sourceDir) = $CheckPar->getListPar();
 my $saveRAM = $CheckPar->getOptWithoutPar('saveRAM');
+my $createSparseFiles = $CheckPar->getOptWithoutPar('createSparseFiles');
+my $blockSize = $CheckPar->getOptWithPar('blockSize');
 $tmpdir = $CheckPar->getOptWithPar('tmpdir');
 
-
 my $prLog;
 my ($prLogKind) = ['A:BEGIN',
 		   'Z:END',
@@ -249,7 +284,36 @@ my ($prLogKind) = ['A:BEGIN',
 		   'E:ERROR',
 		   'S:STATISTIC',
 		   'P:PROGRESS'];
-$prLog = printLog->new('-kind' => $prLogKind);
+$prLog = printLog->new('-kind' => $prLogKind,
+		       '-tmpdir' => $tmpdir);
+
+# check progressReport settings
+my $progressDeltaTime = 0;
+if ($progressReport)
+{
+    my ($count, $t);
+    if ($progressReport =~ /,/)
+    {
+	($count, $t) = $progressReport =~ /\A(.*?),(.*)\Z/;
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["wrong format for option progressReport " .
+				 "time period <$t>"],
+		      '-exit' => 1)
+	    unless &dateTools::checkStr('-str' => $t);
+	$progressDeltaTime = &dateTools::strToSec('-str' => $t);
+    }
+    else
+    {
+	$count = $progressReport;
+    }
+    $prLog->print('-kind' => 'E',
+		  '-str' => ["counter <$count> for progress report " .
+			     "must be a positive integer"],
+		  '-exit' => 1)
+	unless $count =~ /\A\d+\Z/;
+    $progressReport = $count;
+}
+#print "progressReport=<$progressReport>, progressDeltaTime=<$progressDeltaTime>\n";
 
 my (@lt) = (@$linkWith);
 my (@lb) = (@sourceDir);
@@ -412,14 +476,21 @@ if ($saveRAM)
     dbmopen(%md52file, $md52fileFile, 0600);
 }
 my $stat = Statistic->new('-prLog' => $prLog,
-			  '-progressReport' => $progressReport);
+			  '-progressReport' => $progressReport,
+			  '-progressDeltaTime' => $progressDeltaTime);
 
 
 #
 # read all linkWith directores
 #
 my $rrd = recursiveReadDir->new('-dirs' => $linkWith,
+				'-printDepth' => $printDepth,
+				'-printDepthPrlogKind' => 'P',
 				'-prLog' => $prLog);
+$prLog->print('-kind' => 'I',
+	      '-str' => ["start reading linkWith dirs <" .
+			 join('> <', @$linkWith) . '>'])
+    if @$linkWith;
 my ($f, $t);
 while (($f, $t) = $rrd->next())
 {
@@ -436,6 +507,13 @@ while (($f, $t) = $rrd->next())
 	next if exists $inode2md5{$devInode};  # already read
 
 	my $md5 = &::calcMD5($f, $prLog);
+	unless ($md5)
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' =>
+			  ["could not calculate md5 sum of <$f>, skipping"]);
+	    next;
+	}
 	$stat->incrMd5();
 	my $md5size = "$md5-$size";
 	$inode2md5{$devInode} = $md5size;
@@ -455,6 +533,14 @@ while (($f, $t) = $rrd->next())
 	next if exists $inode2md5{$devInode};  # already read
 
 	my $md5 = &::readSymLinkCalcMd5($f, $prLog);
+	unless ($md5)
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' =>
+			  ["could not calculate md5 sum of symlink <$f>," .
+			   " skipping"]);
+	    next;
+	}
 	$stat->incrMd5();
 	my $md5size = "L-$md5-$size";
 	$inode2md5{$devInode} = $md5size;
@@ -471,6 +557,8 @@ $targetDir = &::absolutePath($targetDir);
 #
 # copy / link @sourceDir
 #
+$prLog->print('-kind' => 'I',
+	      '-str' => ["start copying <" . join('> <', @sourceDir) . '>']);
 foreach my $d (@sourceDir)
 {
 #print "---$d---\n";
@@ -495,7 +583,9 @@ foreach my $d (@sourceDir)
 	$stat->incrDir();
     }
 
-    $rrd = recursiveReadDir->new('-dirs' => [$d],,
+    $rrd = recursiveReadDir->new('-dirs' => [$d],
+				 '-printDepth' => $printDepth,
+				 '-printDepthPrlogKind' => 'P',
 				 '-prLog' => $prLog);
     while (($f, $t) = $rrd->next())
     {
@@ -528,11 +618,11 @@ foreach my $d (@sourceDir)
 	elsif ($t eq 'f' or $t eq 'l')  # normal file or symlink
 	{
 	    my ($dev, $inode, $mode, $uid, $gid, $size, $atime,
-		$mtime, $ctime);
+		$mtime, $ctime, $blocks);
 	    if ($t eq 'f')
 	    {
 		($dev, $inode, $mode, $uid, $gid, $size, $atime,
-		$mtime, $ctime) = (stat($f))[0,1,2,4,5,7,8,9,10];
+		$mtime, $ctime, $blocks) = (stat($f))[0,1,2,4,5,7,8,9,10,12];
 	    }
 	    elsif ($t eq 'l')
 	    {
@@ -559,7 +649,11 @@ foreach my $d (@sourceDir)
 
 		    if ($t eq 'f')
 		    {
-			unless (&::copyFile($f, $fnew))
+#			unless (&::copyFile($f, $fnew, $prLog))
+			unless (&::checkSparseAndCopyFiles(
+				       $f, $fnew, $prLog, $size,
+				       $blocks, $createSparseFiles,
+				       $blockSize, $tmpdir))
 			{
 			    $prLog->print('-kind' => 'E',
 					  '-str' =>
@@ -592,11 +686,26 @@ foreach my $d (@sourceDir)
 		if ($t eq 'f')
 		{
 		    my $md5 = &::calcMD5($f, $prLog);
+		    unless ($md5)
+		    {
+			$prLog->print('-kind' => 'E',
+				      '-str' =>
+				      ["could not calculate md5 sum of <$f>, skipping"]);
+			next;
+		    }
 		    $md5size = "$md5-$size";
 		}
 		elsif ($t eq 'l')
 		{
 		    my $md5 = &::readSymLinkCalcMd5($f, $prLog);
+		    unless ($md5)
+		    {
+			$prLog->print('-kind' => 'E',
+				      '-str' =>
+				      ["could not calculate md5 sum of symlink <$f>," .
+				       " skipping"]);
+			next;
+		    }
 		    $md5size = "L-$md5-$size";
 		}
 
@@ -617,7 +726,11 @@ foreach my $d (@sourceDir)
 
 			if ($t eq 'f')
 			{
-			    unless (&::copyFile($f, $fnew))
+#			    unless (&::copyFile($f, $fnew, $prLog))
+			    unless (&::checkSparseAndCopyFiles(
+					   $f, $fnew, $prLog, $size,
+					   $blocks, $createSparseFiles,
+					   $blockSize, $tmpdir))
 			    {
 				$prLog->print('-kind' => 'E',
 					      '-str' =>
@@ -652,7 +765,11 @@ foreach my $d (@sourceDir)
 #print "\tcopy $f =>$fnew\n";
 		    if ($t eq 'f')
 		    {
-			unless (&::copyFile($f, $fnew))
+#			unless (&::copyFile($f, $fnew, $prLog))
+			unless (&::checkSparseAndCopyFiles(
+				       $f, $fnew, $prLog, $size,
+				       $blocks, $createSparseFiles,
+				       $blockSize, $tmpdir))
 			{
 			    $prLog->print('-kind' => 'E',
 					  '-str' =>
@@ -675,13 +792,25 @@ foreach my $d (@sourceDir)
 		    $stat->incrCopy();
 		}
 	    }
-	    chown $uid, $gid, $fnew;
-	    chmod $mode, $fnew;
-	    utime $atime, $mtime, $fnew;
+	    if ($t eq 'l')      # symbolic link
+	    {
+		my $chown =
+		    forkProc->new('-exec' => 'chown',
+				  '-param' => ['-h', "$uid:$gid", $fnew],
+				  '-outRandom' => "$tmpdir/chown-",
+				  '-prLog' => $prLog);
+		$chown->wait();
+	    }
+	    else
+	    {
+		chown $uid, $gid, $fnew;
+		chmod $mode, $fnew;
+		utime $atime, $mtime, $fnew;
+	    }
 	}
 	else   # all file types except 'f' and 'd'
 	{
-	    &::copyDir($f, $fnew, '/tmp/stbuLink-', $prLog, $ignoreErrors);
+	    &::copyDir($f, $fnew, "$tmpdir/stbuLink-", $prLog, $ignoreErrors);
 	    $stat->incrCopy();
 	}
     }
@@ -743,6 +872,59 @@ else
 exit 0;
 
 
+##################################################
+# retunrs 1 on success, 0 if copy fails
+sub checkSparseAndCopyFiles
+{
+    my $source = shift;
+    my $target = shift;
+    my $prLog = shift;
+    my $size = shift;
+    my $blocks = shift;
+    my $createSparseFiles = shift;
+    my $blockSize = shift;
+    my $tmpdir = shift;
+
+    if ($createSparseFiles and  $blocks * $blockSize < $size) # maybe sparse file
+    {
+	my $ret = 1;   # success
+	my $cp = forkProc->new('-exec' => 'cp',
+			       '-param' =>
+			       ["$source", "$target"],
+			       '-outRandom' => "$tmpdir/linkTo-cp-",
+			       '-prLog' => $prLog);
+	$cp->wait();
+	my $out = $cp->getSTDOUT();
+	if (@$out)
+	{
+	    $prLog->print('-kind' => 'W',
+			  '-str' =>
+			  ["copying of <$source> to <$target> reported:",
+			   @$out]);
+	}
+	$out = $cp->getSTDERR();
+	if (@$out)
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' =>
+		  ["copying (with 'cp -r') of <$source> to <$target> reported:",
+			   @$out]);
+	    $ret = 0;
+	}
+	$prLog->print('-kind' => 'I',
+		      '-str' => ["file <$source> copied as sparse file"])
+	    if $ret;
+
+	return $ret;
+    }
+    else
+    {
+	return &::copyFile($source, $target, $prLog);
+    }
+}
+
+
+
 ##################################################
 package Statistic;
 
@@ -753,7 +935,8 @@ sub new
     my $self = {};
 
     my (%params) = ('-prLog'             => undef,
-		    '-progressReport'    => undef
+		    '-progressReport'    => undef,
+		    '-progressDeltaTime' => 0
 		    );
 
     &::checkObjectParams(\%params, \@_, 'Statistic::new',
@@ -767,6 +950,9 @@ sub new
     $self->{'statRead'} = 0;
     $self->{'statMd5'} = 0;
 
+    $self->{'timeProgrReport'} =
+	($self->{'progressDeltaTime'} > 0) ? time : 0;
+
     bless $self, $class;
 }
 
@@ -820,13 +1006,17 @@ sub checkProgressRead
     my $self = shift;
 
     my $n = $self->{'statRead'};
-    if ($self->{'progressReport'} and
-	$n % $self->{'progressReport'} == 0)
+    if (($self->{'progressReport'} and
+	$n % $self->{'progressReport'} == 0) or
+	($self->{'timeProgrReport'} > 0 and
+	 time >= $self->{'timeProgrReport'} + $self->{'progressDeltaTime'}))
     {
 	$self->{'prLog'}->print('-kind' => 'P',
 				'-str' =>
 				["read $n items, calced " .
 				$self->{'statMd5'} . " md5 sums"]);
+	$self->{'timeProgrReport'} = time
+	    if $self->{'timeProgrReport'} > 0;
     }
 }
 
@@ -837,8 +1027,10 @@ sub checkProgress
 
     my $n = $self->{'statCopy'} + $self->{'statLink'} +
 	$self->{'statDir'};
-    if ($self->{'progressReport'} and
-	$n % $self->{'progressReport'} == 0)
+    if (($self->{'progressReport'} and
+	 $n % $self->{'progressReport'} == 0) or
+	($self->{'timeProgrReport'} > 0 and
+	 time >= $self->{'timeProgrReport'} + $self->{'progressDeltaTime'}))
     {
 	$self->{'prLog'}->print('-kind' => 'P',
 				'-str' =>
@@ -846,6 +1038,8 @@ sub checkProgress
 				$self->{'statMd5'} . " md5 sums; " .
 				$self->{'statLink'} . " linked, " .
 				$self->{'statCopy'} . " copied"]);
+	$self->{'timeProgrReport'} = time
+	    if $self->{'timeProgrReport'} > 0;
     }
 }
 
