@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2008-2013)
+#   Copyright (C) Dr. Heinz-Josef Claes (2008-2014)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -106,6 +106,10 @@ require 'storeBackupLib.pl';
 
 my $checkSumFile = '.md5CheckSums';
 
+my $tmpdir = '/tmp';              # default value
+$tmpdir = $ENV{'TMPDIR'} if defined $ENV{'TMPDIR'};
+
+
 =head1 NAME
 
 storeBackupCheckBackup.pl - checks if a file in the backup is missing
@@ -114,8 +118,8 @@ or corrupted
 =head1 SYNOPSIS
 
 	storeBackupCheckBackup.pl -c backupDir [-p number] [-i]
-	      [-w filePrefix] [--lastOfEachSeries]
-	      [--includeRenamedBackups]
+	      [-w filePrefix] [--lastOfEachSeries] 
+	      [--includeRenamedBackups] [-T tmpdir]
 	      [--logFile
 	       [--plusLogStdout] [--suppressTime] [-m maxFilelen]
 	       [[-n noOfOldFiles] | [--saveLogs]]
@@ -163,6 +167,10 @@ It only checks plain files, not special files or symbolic links.
     include renamed backups into the check renamed backups must
     follow the convention <backupDir>-<something>
 
+=item B<--tmpdir>, B<-T>
+
+    directory for temporary files, default is </tmp>
+
 =item B<--wrongFileTables>, B<-w>
 
     write filenames with detected faults in regular files for
@@ -173,6 +181,8 @@ It only checks plain files, not special files or symbolic links.
     /tmp/bugsB-files.missing.txt
     /tmp/bugsB-md5sums.missing.txt
     /tmp/bugsB-md5sums.wrong.txt
+    if you change option tmpdir to something else, this value will be
+    used here instead of /tmp
 
 =item B<--logFile>, B<-l>
 
@@ -198,7 +208,7 @@ It only checks plain files, not special files or symbolic links.
 =item B<--saveLogs>
 
     save log files with date and time instead of deleting the
-    old (with [-noOldFiles])
+    old (with [-noOfOldFiles])
 
 =item B<--compressWith>
 
@@ -209,16 +219,14 @@ It only checks plain files, not special files or symbolic links.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008-2013 by Heinz-Josef Claes (see README)
+Copyright (c) 2008-2014 by Heinz-Josef Claes (see README)
 Published under the GNU General Public License v3 or any later version
 
 =cut
 
-my $Help = join('', grep(!/^\s*$/, `pod2text "$0"`));
-$Help = "cannot find pod2text, see documentation for details\n"
-    unless $Help;
+my $Help = &::getPod2Text($0);
 
-&printVersion(\@ARGV, '-V');
+&printVersion(\@ARGV, '-V', '--version');
 
 my $CheckPar =
     CheckParam->new('-allowLists' => 'no',
@@ -277,6 +285,11 @@ my $CheckPar =
 					    '-quoteEval' => 'yes',
                                             '-default' => 'bzip2',
                                             '-only_if' =>'[logFile]'),
+				Option->new('-name' => 'tmpdir',
+					    '-cl_option' => '-T',
+					    '-cl_alias' => '--tmpdir',
+					    '-default' => $tmpdir),
+
 # hidden options
 # used by storeBackupMount.pl
 				Option->new('-name' => 'writeToNamedPipe',
@@ -309,6 +322,7 @@ my $noOfOldFiles = $CheckPar->getOptWithPar('noOfOldFiles');
 my $saveLogs = $CheckPar->getOptWithPar('saveLogs');
 $saveLogs = $saveLogs ? 'yes' : 'no';
 my $compressWith = $CheckPar->getOptWithPar('compressWith');
+$tmpdir = $CheckPar->getOptWithPar('tmpdir');
 # hidden options
 my $writeToNamedPipe = $CheckPar->getOptWithPar('writeToNamedPipe');
 
@@ -362,21 +376,24 @@ my $prLog1 = printLog->new('-kind' => $prLogKind,
 			   '-maxFilelen' => $maxFilelen,
 			   '-noOfOldFiles' => $noOfOldFiles,
 			   '-saveLogs' => $saveLogs,
-			   '-compressWith' => $compressWith);
+			   '-compressWith' => $compressWith,
+			   '-tmpdir' => $tmpdir);
 
 my $prLog = printLogMultiple->new('-prLogs' => [$prLog1]);
 
 if ($plusLogStdout)
 {
     my $p = printLog->new('-kind' => $prLogKind,
-			  '-filedescriptor', *STDOUT);
+			  '-filedescriptor', *STDOUT,
+			  '-tmpdir' => $tmpdir);
     $prLog->add('-prLogs' => [$p]);
 }
 if ($writeToNamedPipe)
 {
     my $np = printLog->new('-kind' => $prLogKind,
 			   '-file' => $writeToNamedPipe,
-			   '-maxFilelen' => 0);
+			   '-maxFilelen' => 0,
+			   '-tmpdir' => $tmpdir);
     $prLog->add('-prLogs' => [$np]);
 }
 
@@ -481,6 +498,7 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 		$prLog->print('-kind' => 'E',
 			      '-str' =>
 			      ["md5 sum mismatch for <$fn>"]);
+		$main::wft->print("$fn", 'md5Wrong');
 	    }
 	    elsif ($what eq 'errors')
 	    {
@@ -577,7 +595,8 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 
 	    $rcsf = readCheckSumFile->new('-checkSumFile' =>
 					  "$dirToCheck/$checkSumFile",
-					  '-prLog' => $prLog);
+					  '-prLog' => $prLog,
+					  '-tmpdir' => $tmpdir);
 	    $postfix = $rcsf->getInfoWithPar('postfix');
 	    my $writeExcludeLog = $rcsf->getInfoWithPar('writeExcludeLog');
 	    my $logInBackupDir = $rcsf->getInfoWithPar('logInBackupDir');
@@ -596,6 +615,7 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 	    $filesFromMD5CheckSumFile{'.md5BlockCheckSums.bz2'} = 1;
 	    $filesFromMD5CheckSumFile{'.md5CheckSums.bz2'} = 1;
 	    $filesFromMD5CheckSumFile{'.md5CheckSums'} = 1;
+	    $filesFromMD5CheckSumFile{'.md5CheckSums.Finished'} = 1;
 	    $filesFromMD5CheckSumFile{'.md5CheckSums.info'} = 1;
 	    $filesFromMD5CheckSumFile{'.storeBackupLinks'} = 1;
 	    $filesFromMD5CheckSumFile{'.storeBackup.notSaved.bz2'} = 1
@@ -699,7 +719,7 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 		}
 
 #print "-6-\n";
-		my $tmpName = &::uniqFileName("/tmp/storeBackup-block.");
+		my $tmpName = &::uniqFileName("$tmpdir/storeBackup-block.");
 		$parFork->add_noblock('-function' => \&checkMD5,
 				      '-funcPar' =>
 				      [$dirToCheck, $filename, $md5sum, $compr,
@@ -915,7 +935,7 @@ sub checkMD5
 		    pipeFromFork->new('-exec' => $uncompr,
 				      '-param' => \@uncomprPar,
 				      '-stdin' => $f,
-				      '-outRandom' => '/tmp/stbuPipeFrom10-',
+				      '-outRandom' => "$tmpdir/stbuPipeFrom10-",
 				      '-prLog' => $prLog);
 	    }
 
@@ -973,7 +993,7 @@ sub checkMD5
 		pipeFromFork->new('-exec' => 'bzip2',
 				  '-param' => ['-d'],
 				  '-stdin' => "$f/.md5BlockCheckSums.bz2",
-				  '-outRandom' => '/tmp/stbuPipeFrom11-',
+				  '-outRandom' => "$tmpdir/stbuPipeFrom11-",
 				  '-prLog' => $prLog);
 
 	    my $l;
@@ -990,8 +1010,7 @@ sub checkMD5
 				  '-str' =>
 				  ["strange line in <$f/.md5BlockCheckSums.bz2> " .
 				   "in line " . $fileIn->get('-what' => 'lineNr') .
-				   ":", "\t<$l>"],
-				  '-exit' => 1)
+				   ":", "\t<$l>"]);
 		}
 
 		if (-e "$dirToCheck/$l_f")
@@ -1052,7 +1071,7 @@ sub checkMD5
 			pipeFromFork->new('-exec' => $uncompr,
 					  '-param' => \@uncomprPar,
 					  '-stdin' => "$f/$entry",
-					  '-outRandom' => '/tmp/stbuPipeFrom12-',
+					  '-outRandom' => "$tmpdir/stbuPipeFrom12-",
 					  '-prLog' => $prLog);
 		}
 		else           # block not compressed
