@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2003-2012)
+#   Copyright (C) Dr. Heinz-Josef Claes (2003-2014)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -152,6 +152,10 @@ storeBackupDel.pl - this program deletes backups created by storeBackup
 
     lock file, if exists, new instances will finish if
     an old is already running, default is $lockFile
+    this type of lock files does not work across multiple servers
+    and is not designed to separate storeBackup.pl and
+    storeBackupUpdateBackup.pl or any other storeBackup
+    process in a separate PID space
 
 =item B<--doNotDelete>
 
@@ -284,7 +288,7 @@ storeBackupDel.pl - this program deletes backups created by storeBackup
 =item B<--saveLogs>
 
     save log files with date and time instead of deleting the
-    old (with [-noOldFiles])
+    old (with [-noOfOldFiles])
 
 =item B<--compressWith>
 
@@ -295,16 +299,14 @@ storeBackupDel.pl - this program deletes backups created by storeBackup
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003-2012 by Heinz-Josef Claes (see README).
+Copyright (c) 2003-2014 by Heinz-Josef Claes (see README).
 Published under the GNU General Public License v3 or any later version
 
 =cut
 
-my $Help = join('', grep(!/^\s*$/, `pod2text $0`));
-$Help = "cannot find pod2text, see documentation for details\n"
-    unless $Help;
+my $Help = &::getPod2Text($0);
 
-&printVersion(\@ARGV, '-V');
+&printVersion(\@ARGV, '-V', '--version');
 
 my $startDate = dateTools->new();
 
@@ -443,7 +445,12 @@ my $CheckPar =
 					    '-cl_option' => '--today',
 					    '-cf_key' => 'today',
 					    '-hidden' => 'yes',
-					    '-param' => 'yes')
+					    '-param' => 'yes'),
+# used by storeBackupMount.pl
+				Option->new('-name' => 'writeToNamedPipe',
+					    '-cl_option' => '--writeToNamedPipe',
+					    '-param' => 'yes',
+					    '-hidden' => 'yes')
 				]
 		    );
 
@@ -484,10 +491,11 @@ my $maxFilelen = $CheckPar->getOptWithPar('maxFilelen');
 my $noOfOldFiles = $CheckPar->getOptWithPar('noOfOldFiles');
 my $saveLogs = $CheckPar->getOptWithoutPar('saveLogs') ? 'yes' : 'no';
 my $compressWith = $CheckPar->getOptWithPar('compressWith');
-
+# hidden options
 my $printAll = $CheckPar->getOptWithoutPar('printAll');
 $print = 1 if $printAll;
 my $todayOpt = $CheckPar->getOptWithPar('todayOpt');  # format like
+my $writeToNamedPipe = $CheckPar->getOptWithPar('writeToNamedPipe');
                                                       # backup dir name
 
 if ($print)
@@ -521,6 +529,17 @@ if ($plusLogStdout)
 			  '-filedescriptor', *STDOUT);
     $prLog->add('-prLogs' => [$p]);
 }
+if ($writeToNamedPipe)
+{
+    my $np = printLog->new('-kind' => \@kind,
+			   '-file' => $writeToNamedPipe,
+			   '-maxFilelen' => 0);
+    $prLog->add('-prLogs' => [$np]);
+}
+
+(@main::cleanup) = ($prLog, undef);
+$SIG{INT} = \&cleanup;
+$SIG{TERM} = \&cleanup;
 
 $prLog->print('-kind' => 'A',
 	      '-str' =>
@@ -691,3 +710,14 @@ $prLog->print('-kind' => 'Z',
 
 
 exit 0;
+
+
+##################################################
+# package printLogMultiple needs this function
+sub cleanup
+{
+    my $signame = shift;
+    my $exit = (shift() || 1);
+
+    exit $exit;
+}
