@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2008-2012)
+#   Copyright (C) Dr. Heinz-Josef Claes (2008-2014)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -99,6 +99,9 @@ require 'storeBackupLib.pl';
 
 my $checkSumFile = '.md5CheckSums';
 
+my $tmpdir = '/tmp';              # default value
+$tmpdir = $ENV{'TMPDIR'} if defined $ENV{'TMPDIR'};
+
 =head1 NAME
 
 storeBackupSearch.pl - locates different versions of a file saved with storeBackup.pl.
@@ -109,7 +112,7 @@ storeBackupSearch.pl - locates different versions of a file saved with storeBack
 
 	storeBackupSearch.pl -b backupDirDir [-f configFile]
 	      [-s rule]  [--absPath] [-w file] [--parJobs number]
-	      [-d level] [--once] [--print] [backupRoot . . .]
+	      [-d level] [--once] [--print] [-T tmpdir] [backupRoot . . .]
 
 =head1 DESCRIPTION
 
@@ -161,6 +164,10 @@ You need some basic understanding of linux and perl to use it.
 
     show every file found only once (depending on md5 sum)
 
+=item B<--tmpdir>, B<-T>
+
+    directory for temporary files, default is </tmp>
+
 =item backupRoot
 
     Root directories of backups where to search relative
@@ -171,14 +178,12 @@ You need some basic understanding of linux and perl to use it.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008-2012 by Heinz-Josef Claes (see README).
+Copyright (c) 2008-2014 by Heinz-Josef Claes (see README).
 Published under the GNU General Public License v3 or any later version
 
 =cut
 
-my $Help = join('', grep(!/^\s*$/, `pod2text $0`));
-$Help = "cannot find pod2text, see documentation for details\n"
-    unless $Help;
+my $Help = &::getPod2Text($0);
 
 my $templateConfigFile = <<EOC;
 # configuration file for storeBackupSearch.pl, version $main::STOREBACKUPVERSION
@@ -222,6 +227,9 @@ my $templateConfigFile = <<EOC;
 # root directory of backup relative to backupDir directory
 ;backupRoot=
 
+# directory for temporary file, default is /tmp
+;tmpDir=
+
 # write result with absolute path names
 # default is 'no', possible values are 'yes' and 'no'
 ;absPath=
@@ -242,7 +250,7 @@ EOC
     ;
 
 
-&printVersion(\@ARGV, '-V');
+&printVersion(\@ARGV, '-V', '--version');
 
 my $CheckPar =
     CheckParam->new('-configFile' => '-f',
@@ -298,6 +306,11 @@ my $CheckPar =
 					    '-cf_noOptSet' => ['yes', 'no']),
 				Option->new('-name' => 'print',
 					    '-cl_option' => '--print'),
+				Option->new('-name' => 'tmpdir',
+					    '-cl_option' => '-T',
+					    '-cl_alias' => '--tmpdir',
+					    '-cf_key' => 'tmpDir',
+					    '-default' => $tmpdir),
 				Option->new('-name' => 'backupRoot',
 					    '-cf_key' => 'backupRoot',
 					    '-param' => 'yes'),
@@ -329,6 +342,7 @@ my $writeAbsPath = $CheckPar->getOptWithoutPar('writeAbsPath');
 my $parJobs = $CheckPar->getOptWithPar('parJobs');
 my $debug = $CheckPar->getOptWithPar('debug');
 my $once = $CheckPar->getOptWithoutPar('once');
+$tmpdir = $CheckPar->getOptWithPar('tmpdir');
 my (@backupRoot) = $CheckPar->getListPar();
 
 my $printAll = $CheckPar->getOptWithoutPar('printAll');
@@ -381,7 +395,9 @@ if ($print)
 }
 
 my $prLog = printLog->new('-kind' => ['I:INFO', 'W:WARNING', 'E:ERROR',
-				      'S:STATISTIC', 'D:DEBUG', 'V:VERSION']);
+				      'S:STATISTIC', 'D:DEBUG', 'V:VERSION'],
+			  '-tmpdir' => $tmpdir);
+$main::__prLog = $prLog;   # used in rules
 $prLog->fork($req);
 
 $prLog->print('-kind' => 'V',
@@ -508,7 +524,10 @@ foreach $dirToSearch (@dirsToSearch)
 				 "<$dirToSearch> ... skipping"]);
 	next;
     }
-    if (-f "$dirToSearch/$checkSumFile.notFinished")
+#    if (-f "$dirToSearch/$checkSumFile.notFinished")
+    unless (&::checkIfBackupWasFinished('-backupDir' => "$dirToSearch",
+					    '-prLog' => $prLog,
+					'-count' => 40))
     {
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["backup <$dirToSearch> not finished" .
@@ -525,7 +544,8 @@ foreach $dirToSearch (@dirsToSearch)
 
     my $rcsf =
 	readCheckSumFile->new('-checkSumFile' => "$dirToSearch/$checkSumFile",
-			      '-prLog' => $prLog);
+			      '-prLog' => $prLog,
+			      '-tmpdir' => $tmpdir);
 #    my $meta = $rcsf->getMetaValField();
 #    my $postfix = ($$meta{'postfix'})->[0];    # postfix for compression
     my $postfix = $rcsf->getInfoWithPar('postfix');
@@ -598,7 +618,7 @@ foreach $dirToSearch (@dirsToSearch)
 		push @lineBuffer, $l;
 	    }
 	    $jobToDo = @lineBuffer;
-	    my $tmpName = &::uniqFileName("/tmp/storeBackupSearch-");
+	    my $tmpName = &::uniqFileName("/$tmpdir/storeBackupSearch-");
 
 	    if ($jobToDo)
 	    {
