@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2008-2013)
+#   Copyright (C) Dr. Heinz-Josef Claes (2008-2014)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -118,7 +118,8 @@ storeBackupUpdateBackup.pl - updates / finalizes backups created by storeBackup.
 	      [--print] [--verbose] [--debug] [--lockFile] [--noCompress]
 	      [--progressReport number] [--checkOnly] [--copyBackupOnly]
 	      [--dontCopyBackup] [-A archiveDurationDeltaCache]
-	      [--dontDelInDeltaCache]
+	      [--dontDelInDeltaCache] [--createNewSeries]
+	      [--noWarningDiffSeriesInBackupCopy]
 	      [--logFile
 	       [--plusLogStdout] [--suppressTime] [-m maxFilelen]
 	       [[-n noOfOldFiles] | [--saveLogs]]
@@ -173,6 +174,10 @@ storeBackupUpdateBackup.pl - updates / finalizes backups created by storeBackup.
     If set to the same file as in storeBackup it will
     prevent $prog from running in parallel
     to storeBackup, default is $lockFile
+    this type of lock files does not work across multiple servers
+    and is not designed to separate storeBackup.pl and
+    storeBackupUpdateBackup.pl or any other storeBackup
+    process in a separate PID space
 
 =item B<--noCompress>
 
@@ -207,6 +212,13 @@ storeBackupUpdateBackup.pl - updates / finalizes backups created by storeBackup.
 =item B<--dontDelInDeltaCache>
     do not delete any backup in deltaCache
 
+=item B<--createNewSeries> B<-C>
+
+    Automatically create new series in deltaCache and the
+    replication directory / directories. This is especially useful
+    if you use wildcards to specify series name which should be
+    added 'on the fly'
+
 =item B<--progressReport>
 
     print progress report:
@@ -221,6 +233,12 @@ storeBackupUpdateBackup.pl - updates / finalizes backups created by storeBackup.
 
 =back
 
+=item B<--noWarningDiffSeriesInBackupCopy>, B<-N>
+
+    do not write a warning if there is a different set of series
+    inside the backup copy (replication directory) than in the
+    actual deltaCache
+
 =item B<--logFile>, B<-l>
 
     logFile, Default: stdout
@@ -245,7 +263,7 @@ storeBackupUpdateBackup.pl - updates / finalizes backups created by storeBackup.
 =item B<--saveLogs>
 
     save log files with date and time instead of deleting the
-    old (with [-noOldFiles])
+    old (with [-noOfOldFiles])
 
 =item B<--compressWith>
 
@@ -266,16 +284,14 @@ storeBackupUpdateBackup.pl - updates / finalizes backups created by storeBackup.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008-2013 by Heinz-Josef Claes.
+Copyright (c) 2008-2014 by Heinz-Josef Claes.
 Published under the GNU General Public License v3 or any later version
 
 =cut
 
-my $Help = join('', grep(!/^\s*$/, `pod2text "$0"`));
-$Help = "cannot find pod2text, see documentation for details\n"
-    unless $Help;
+my $Help = &::getPod2Text($0);
 
-&printVersion(\@ARGV, '-V');
+&printVersion(\@ARGV, '-V', '--version');
 
 my $startDate = dateTools->new();
 my $CheckPar =
@@ -319,6 +335,10 @@ my $CheckPar =
                                 Option->new('-name' => 'dontDelInDeltaCache',
 					    '-cl_option' => '--dontDelInDeltaCache',
 					    '-only_if' => 'not [interactive]'),
+				Option->new('-name' => 'createNewSeries',
+					    '-cl_option' => '-C',
+					    '-only_if' => 'not [interactive]',
+					    '-cl_alias' => '--createNewSeries'),
 				Option->new('-name' => 'progressReport',
 					    '-cl_option' => '--progressReport',
 					    '-default' => 0,
@@ -329,8 +349,12 @@ my $CheckPar =
 					    '-cl_alias' => '--verbose'),
 				Option->new('-name' => 'debug',
 					    '-cl_option' => '--debug',
-					    '-cl_option' => '-d',
+					    '-cl_alias' => '-d',
 					    '-only_if' => 'not [interactive]'),
+				Option->new('-name' => 'noWarningDiffSeriesInBackupCopy',
+					    '-cl_option' => '-N',
+					    '-only_if' => 'not [interactive]',
+					    '-cl_alias' => '--noWarningDiffSeriesInBackupCopy'),
 				Option->new('-name' => 'logFile',
 					    '-cl_option' => '-l',
 					    '-cl_alias' => '--logFile',
@@ -398,7 +422,10 @@ my $copyBackupOnly = $CheckPar->getOptWithoutPar('copyBackupOnly');
 my $dontCopyBackup = $CheckPar->getOptWithoutPar('dontCopyBackup');
 $archiveDurationDeltaCache = $CheckPar->getOptWithPar('archiveDurationDeltaCache');
 my $dontDelInDeltaCache = $CheckPar->getOptWithoutPar('dontDelInDeltaCache');
+my $createNewSeries = $CheckPar->getOptWithoutPar('createNewSeries');
 my $progressReport = $CheckPar->getOptWithPar('progressReport');
+my $noWarningDiffSeriesInBackupCopy =
+    $CheckPar->getOptWithoutPar('noWarningDiffSeriesInBackupCopy');
 my $logFile = $CheckPar->getOptWithPar('logFile');
 my $plusLogStdout = $CheckPar->getOptWithoutPar('plusLogStdout');
 my $withTime = not $CheckPar->getOptWithoutPar('suppressTime');
@@ -634,7 +661,7 @@ $prLog->print('-kind' => 'A',
 $prLog->print('-kind' => 'V',
 	      '-str' => ["storeBackupUpdateBackup.pl, $main::STOREBACKUPVERSION"]);
 
-::checkLockFile($lockFile, $prLog);
+&::checkLockFile($lockFile, $prLog);
 
 exit 1
     unless &deleteOldBackupDirs::checkTimeScaleFormat('archiveDurationDeltaCache',
@@ -648,7 +675,16 @@ if (-e "$backupDir/$baseTreeConf" and not $dontCopyBackup)
 # read storeBackupBaseTree.conf in $backupDir
     my ($backupTreeName, $backupType, $seriesToDistribute);
     ($backupTreeName, $backupType, $seriesToDistribute, $deltaCache) =
-	&::readBackupDirBaseTreeConf("$backupDir/$baseTreeConf", $prLog);
+	&::readBackupDirBaseTreeConf("$backupDir/$baseTreeConf",
+				     &::absolutePath($backupDir), $prLog);
+
+#print "-1----------------=======------------\n";
+    # replace wildcards in replicated backup
+    my (@readSeries) = &::evalExceptionList_PlusMinus($seriesToDistribute,
+					    &::absolutePath($backupDir),
+				      "$backupTreeName series", 'series', 1,
+					    undef, 1, $prLog);
+#print "-1.1------readSeries=@readSeries\n";
 
 #print "backupTreeName = <$backupTreeName>\n";
 #print "backupType = <$backupType>\n";
@@ -671,11 +707,63 @@ if (-e "$backupDir/$baseTreeConf" and not $dontCopyBackup)
 
     if ($backupType eq 'copy')
     {
-	$autorepairError = 0;
+#print "-2----------------=======------------\n";
+	# build hash with list of series in replication directory
+	my (%readSeries);
+#print "readSeries=<@readSeries>\n";
+	foreach my $r (@readSeries)
+	{
+#print "$r\n";
+	    $readSeries{$r} = 1;
+	}
+
 	# read deltaCache.conf in $deltaCache
 	my $cscFile = "$deltaCache/$deltaCacheConf";
+	my (@bc) = &::readDeltaCacheConf($cscFile, $deltaCache, 1, $prLog);
 
-	my (@bc) = &::readDeltaCacheConf($cscFile, $prLog);
+#print "-2.5----------------=======------------\n";
+	# generate list with existing series in Delta Cache
+	my (@listSeriesExistInDelta) = ();
+	foreach my $bc (@bc)
+	{
+#print "===> $bc[0][0] --------- $backupTreeName\n";
+	    if ($bc[0][0] eq $backupTreeName)
+	    {
+		(@listSeriesExistInDelta) = (@$bc[1..@$bc-1]);
+	    }
+#print "-3----------+Delta Cache Series++++++@listSeriesExistInDelta++\n";
+	}
+	# generate missing series directories in replication directory
+	# This has to be done 'on the fly' also when using wildcards in series names
+	my $createSeries = 0;
+	foreach my $d (@listSeriesExistInDelta)
+	{
+#print "3.5 -- <$d> -- ", (defined $readSeries{$d}), "\n";
+	    if ($createNewSeries and not defined $readSeries{$d})
+	    {
+#print "3.6-- <$backupDir/$d>\n";
+		exit 1
+		    unless &::makeDirPath("$backupDir/$d", $prLog);
+		$createSeries = 1;
+	    }
+	}
+#print "-4----------------=======------------\n";
+#print "-4.1------seriesToDistribute=@$seriesToDistribute\n";
+	if ($createSeries)
+	{
+	    # replace wildcards in replicated backup
+	    (@readSeries) = &::evalExceptionList_PlusMinus($seriesToDistribute,
+						    &::absolutePath($backupDir),
+						    "$backupTreeName series", 'series',
+						    1, undef, 1, $prLog);
+	}
+	$seriesToDistribute =\@readSeries;
+#print "-4.2------seriesToDistribute=@$seriesToDistribute\n";
+
+	$autorepairError = 0;
+#	# read deltaCache.conf in $deltaCache
+#	my $cscFile = "$deltaCache/$deltaCacheConf";
+#	my (@bc) = &::readDeltaCacheConf($cscFile, $deltaCache, 1, $prLog);
 
 	# check consistency of backupCopy and deltaCache configuration
 	my $foundBackupTreeNameFlag = 0;
@@ -706,18 +794,19 @@ if (-e "$backupDir/$baseTreeConf" and not $dontCopyBackup)
 		}
 		foreach $s (@$seriesToDistribute)
 		{
+#print "---removing series <$s>\n";
 		    if (exists $s{$s})
 		    {
 			delete $s{$s};
 		    }
 		    else
 		    {
-			$prLog->print('-kind' => 'E',
+			$prLog->print('-kind' => 'W',
 				      '-str' =>
 				      ["$cscFile: series <$s> is missing in <" .
 				       $$bc[0] . ">, defined in " .
-				       "$backupDir/$baseTreeConf"],
-				      '-exit' => 1);
+				       "$backupDir/$baseTreeConf"])
+			    unless $noWarningDiffSeriesInBackupCopy;
 		    }
 
 		}
@@ -726,7 +815,9 @@ if (-e "$backupDir/$baseTreeConf" and not $dontCopyBackup)
 			      '-str' =>
 			      ["$backupDir/$baseTreeConf series <" .
 			       join('><', sort keys %s) . "> missing in <" .
-			       $$bc[0] . ">, defined in $cscFile"],
+			       $$bc[0] . ">, defined in $cscFile",
+			       "use option --createNewSeries if you want " .
+			       "missing series to be created automatically"],
 			      '-exit' => 1)
 		    if scalar keys %s;
 	    }
@@ -811,6 +902,8 @@ if (-e "$backupDir/$baseTreeConf" and not $dontCopyBackup)
     }
     elsif ($backupType eq 'master')
     {
+	$seriesToDistribute = \@readSeries;
+
 	my $s;
 	foreach $s (@$seriesToDistribute)
 	{
@@ -1005,7 +1098,15 @@ while (((@lateLinkDirs) = $allLinks->getAllDirsWithLateLinks()) > 0)
 #print "<$d> ==> <$aktBackupDir> <$aktSeries> <$aktBackup>\n";
 
 	    my ($backupTreeName, $backupType, $seriesToDistribute, $deltaCache) =
-		&::readBackupDirBaseTreeConf("$backupDir/$baseTreeConf", $prLog);
+		&::readBackupDirBaseTreeConf("$backupDir/$baseTreeConf",
+					     &::absolutePath($backupDir), $prLog);
+	    # replace wildcards
+	    my (@readSeries) = &::evalExceptionList_PlusMinus($seriesToDistribute,
+						    &::absolutePath($backupDir),
+					    "$backupTreeName series", 'series', 0,
+						    undef, 1, $prLog);
+	    $seriesToDistribute =\@readSeries;
+
 	    my $found = 0;
 	    foreach my $s (@$seriesToDistribute)
 	    {
@@ -1276,8 +1377,8 @@ if (-e "$backupDir/$baseTreeConf" and $deltaCache)
 	my $dirs =
 	    allStoreBackupSeries->new('-rootDir' => "$cspb/$s",
 				      '-checkSumFile' => $checkSumFile,
-				      '-prLog' => $prLog,
-				      '-absPath' => 0);
+				      '-prLog' => $prLog);
+#				      '-absPath' => 0);
 	my (@dirs) = $dirs->getAllFinishedDirs();   # oldest first
 	my $d;
 	foreach $d (@dirs)
@@ -1532,7 +1633,12 @@ sub updateBackupDir
 		my $buffer;
 		while (sysread(FILEIN, $buffer, 1025))
 		{
-		    $bz->syswrite($buffer);
+		    unless ($bz->syswrite($buffer))
+		    {
+			$prLog->print('-kind' => 'E',
+				      '-str' =>
+				      ["writing to <$file$comprPostfix> failed"]);
+		    }
 		}
 		$bz->flush();
 		$bz->eof();
@@ -1683,7 +1789,7 @@ sub updateBackupDir
 	    else
 	    {
 		# copy file
-                unless (::copyFile("$existingFile", "$newLink"))
+                unless (&::copyFile("$existingFile", "$newLink", $prLog))
                 {
                     $prLog->print('-kind' => 'E',
                                   '-str' => ["could not link/copy " .
@@ -1865,7 +1971,7 @@ sub updateBackupDir
 	    if (not -l $file and not -e $file)
 	    {
 		$prLog->print('-kind' => 'E',
-			      '-str' => ["cannot acces <$file>"]);
+			      '-str' => ["cannot access <$file>"]);
 		next;
 	    }
 	    $noFiles++;
@@ -1956,7 +2062,7 @@ sub updateBackupDir
 		unless (-e $file)
 		{
 		    $prLog->print('-kind' => 'E',
-				  '-str' => ["cannot acces <$file>"]);
+				  '-str' => ["cannot access <$file>"]);
 		    next;
 		}
 		chown $uid, $gid, $file;
