@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2002-2012)
+#   Copyright (C) Heinz-Josef Claes (2002-2014)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -53,6 +53,9 @@
 #my $configFile = $CheckPar->getOptWithPar('confFile');
 #my $verbose = $CheckPar->getOptWithoutPar('verbose');
 #$CheckPar->print();
+#my (@list) = $CheckPar->getListPar();
+#my numberListPar = $CheckPar->getNoListPar();
+#my (@list) = $CheckPar->getOptOrder();
 #
 ######################################################################
 # Options of Option::new
@@ -471,7 +474,7 @@ sub splitQuotedLine
     $l =~ s/\\\{/$ob/g;    # replace \{ with $ob
     $l =~ s/\\\}/$cb/g;    # replace \} with $cb
     $l =~ s/\\\'/$sq/g;    # replace \' with $sq
-    $l =~ s/\\\"/$dq/g;    # replace \" with $sq
+    $l =~ s/\\\"/$dq/g;    # replace \" with $dq
 
     my (@l);
 #print "l = <$l>, doNotReplaceEnvVar = <$doNotReplaceEnvVar>\n";
@@ -706,12 +709,13 @@ sub check
     }
 
     my ($arg, @parList, $next, $aktOpt, %optWithPar,
-	%optWithoutPar, %unset, $i);
+	@optOrder, %optWithoutPar, %unset, $i);
 
     # Ergebnisse merken
     $self->{'Erg'}{'withPar'} = \%optWithPar;
     $self->{'Erg'}{'withoutPar'} = \%optWithoutPar;
     $self->{'Erg'}{'listPar'} = \@parList;
+    $self->{'Erg'}{'optOrder'} = \@optOrder;
 
     $next = 'unknown';
     my $i;
@@ -751,6 +755,7 @@ sub check
 	    $arg = $o->get('-cl_option');
 	    $o->set('-used' => 'yes');
 	    my $name = $o->get('-name');
+	    push @optOrder, $name;
 
 	    if ($o->get('-param') eq 'yes')
 	    {
@@ -1300,6 +1305,16 @@ sub getNoListPar
     return scalar(@{$self->{'Erg'}{'listPar'}});
 }
 
+########################################
+# order of options in @ARGV, if one option is used multiple times
+# it's also multiple times in the list
+sub getOptOrder
+{
+    my $self = shift;
+
+    return @{$self->{'Erg'}{'optOrder'}};
+}
+
 
 ######################################################################
 # Iterator f"ur die List-Parameter
