@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2001-2013)
+#   Copyright (C) Heinz-Josef Claes (2001-2014)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -24,6 +24,7 @@ use Digest::MD5 qw(md5_hex);
 use Fcntl qw(O_RDWR O_CREAT);
 use Fcntl ':mode';
 use POSIX;
+use Cwd 'abs_path';
 
 require 'prLog.pl';
 require 'forkProc.pl';
@@ -40,6 +41,7 @@ sub copyFile
 {
     my $source = shift;
     my $target = shift;
+    my $prLog = shift;
 
     local *SOURCE;
     local *TARGET;
@@ -54,7 +56,12 @@ sub copyFile
     my $buffer;
     while (sysread(SOURCE, $buffer, 10*1024**2))
     {
-	syswrite(TARGET, $buffer);
+	unless (syswrite(TARGET, $buffer))
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' => [$!]);
+	    return 0;
+	}
     }
 
     close(TARGET);
@@ -238,76 +245,23 @@ sub splitFileDir
 sub absolutePath
 {
     my $dir = shift;
+    my $dirSave = $dir;
 
-    return undef unless $dir;
+    return undef unless $dir and -e $dir;
 
     # First look up in the hash table if that's already calculated
     return $main::absolutePathCache{$dir}
         if exists $main::absolutePathCache{$dir};
-    my $dirSave = $dir;
-
-    # if file itself is a hard link, follow the hard link(s) to something real
-    my $file = '';
-
-
-    $dir = $ENV{'PWD'} . "/$dir" unless ($dir =~ /^\//); # relative path
-
-    if (-e $dir)
-    {
-	$file = $dir;
-	while (-l $file)
-	{
-	    my $link = readlink($file);
-
-	    if (substr($link, 0, 1) ne "/")
-	    {
-		$file =~ s/[^\/]+$/$link/;
-	    }
-	    else
-	    {
-		$file = $link;
-	    }
-	}
-
-	($dir, $file) = &splitFileDir($file);
-	$file = "/$file";
-    }
-
-    my $ret = (-e "$dir$file") ? "$dir$file" : undef;
-    $ret =~ s/\/+/\//g;                # // -> /
-    while ($ret =~ s/\/\.\//\//g) {}   # /./ -> /
-    $ret =~ s/\A(.+?)\/+\Z/$1/;        # remove trailing /
-    $ret =~ s/\A\.\///;                # ./xxx -> xxx
-    $ret =~ s/\/\.\Z//;                # xxx/. -> xxx
-
-    # eliminate /../
-    my (@ret) = split(/\//, $ret);
-    if (@ret)
-    {
-	my (@rev) = reverse(@ret);
-	(@ret) = ();
-	my ($i, $j);
-	for ($i = $j = 0 ; $i < @rev ; $i++, $j++)
-	{
-	    if ($rev[$i] eq '..')
-	    {
-		$j++;
-	    }
-	    else
-	    {
-		push @ret, $rev[$j] if ($j < @rev);
-	    }
-	}
-	$ret = join('/', reverse @ret);
-    }
 
     if (@main::absolutePathCache > 100)
     {
 	my $del = shift @main::absolutePathCache;   # delete oldest
 	delete $main::absolutePathCache{$del};
     }
+    my $ret = abs_path($dir);
     push @main::absolutePathCache, $ret;
-    return $main::absolutePathCache{$dirSave} = $ret;
+    $main::absolutePathCache{$dirSave} = $ret;
+    return $ret;
 }
 
 
@@ -516,6 +470,12 @@ sub checkLockFile
 			  '-str' => ["removing old lock file of process <$pid>"]
 			  );
 	}
+	unless (unlink $lockFile)
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' => ["cannot remove lock file <$lockFile"],
+			  '-exit' => 1);
+	}
     }
 
     $prLog->print('-kind' => 'I',
@@ -540,6 +500,9 @@ sub calcMD5
     local *FROM;
     unless (sysopen(FROM, $filename, O_RDONLY))
     {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot open <$filename> in calcMD5"])
+	    if $prLog;
 	return undef;
     }
     my $md5 = Digest::MD5->new();
@@ -579,6 +542,7 @@ sub createArchiveFromFile
     my $backupDir = shift;
     my $archiver = shift;
     my $prLog = shift;
+    my $tmpdir = shift;
 
     my $prog = $main::fileTypeArchiver{$archiver}{'prog'};
     my $opts = $main::fileTypeArchiver{$archiver}{'createOpts'}; # \@list
@@ -591,7 +555,7 @@ sub createArchiveFromFile
 				'-param' => $opts,
 				'-workingDir' => $pathToSourceFile,
 				'-stdout' => "$backupDir/$filename",
-				'-stderr' => '/tmp/stbuPipeArchive-',
+				'-stderr' => "$tmpdir/stbuPipeArchive-",
 				'-delStdout' => 'no',
 				'-prLog' => $prLog);
 	$tar->wait();
@@ -610,7 +574,7 @@ sub createArchiveFromFile
 				   '-param' => $opts,
 				   '-workingDir' => $pathToSourceFile,
 				   '-stdout' => "$backupDir/$filename",
-				   '-stderr' => '/tmp/stbuPipeArchive-',
+				   '-stderr' => "$tmpdir/stbuPipeArchive-",
 				   '-delStdout' => 'no',
 				   '-prLog' => $prLog);
 	$cpio->print("$f\n");
@@ -637,6 +601,7 @@ sub extractFileFromArchive
     my $backupDir = shift;
     my $archiver = shift;
     my $prLog = shift;
+    my $tmpdir = shift;
 
 #print "+++extractFileFromArchive:+$filename+$restoreDir+$backupDir+$archiver+\n";
     my $prog = $main::fileTypeArchiver{$archiver}{'prog'};
@@ -650,7 +615,7 @@ sub extractFileFromArchive
 	my $tar = forkProc->new('-exec' => $prog,
 				'-param' => $opts,
 				'-workingDir' => $pathToRestoreDir,
-				'-outRandom' => '/tmp/stbuPipeArchive-',
+				'-outRandom' => "$tmpdir/stbuPipeArchive-",
 				'-prLog' => $prLog);
 	$tar->wait();
 	my $out = $tar->getSTDOUT();
@@ -669,7 +634,7 @@ sub extractFileFromArchive
 				 '-param' => $opts,
 				 '-workingDir' => $pathToRestoreDir,
 				 '-stdin' => "$backupDir/$filename",
-				 '-outRandom' => '/tmp/stbuPipeArchive-',
+				 '-outRandom' => "$tmpdir/stbuPipeArchive-",
 				 '-prLog' => $prLog);
 	$cpio->wait();
 	my $out = $cpio->getSTDOUT();
@@ -691,6 +656,125 @@ sub extractFileFromArchive
 }
 
 
+############################################################
+# helper functions for &main::MARK_DIR and &main::MARK_DIR_REC
+# problem is to store 2 dimensional hash with dbm files (saveRAM)
+# simulates $a{$b}{$c} = 1 or $a{$b}{$d} = 1
+# with $a{$b} = 'SEP$cSEP$aSEP' where SEPerator is \000
+sub addToStr
+{
+    my $hash = shift;
+    my $k1 = shift;
+    my $k2 = shift;
+
+    if (exists $$hash{$k1})
+    {
+	if (index($$hash{$k1}, "\000$k2\000") < 0)  # not found
+	{
+	    $$hash{$k1} = $$hash{$k1} . "$k2\000";
+	}
+    }
+    else
+    {
+	$$hash{$k1} = "\000$k2\000";
+    }
+}
+
+########################################
+sub existStr
+{
+    my $hash = shift;
+    my $k1 = shift;
+    my $k2 = shift;
+
+#    print "\thash = ", $$hash{$k1}, "\n";
+#    print "\t\t $k1 exists\n" if (exists $$hash{$k1});
+
+    if (exists $$hash{$k1} and index($$hash{$k1}, "\000$k2\000") >= 0)
+    {
+	return 1;
+    }
+    else
+    {
+	return 0;
+    }
+}
+
+
+############################################################
+# creates sparse file with specified size
+# deletes existing file
+# returns 1 if success / returns 0 if no success
+sub createSparseFile
+{
+    my $file = shift;
+    my $size = shift;
+    my $prLog = shift;
+
+    if (-e $file)
+    {
+	return 0 if unlink $file != 1;
+    }
+
+    unless (sysopen(TO, $file, O_CREAT | O_WRONLY))
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot create sparse file <$file>"]);
+	return 0;
+    }
+
+    sysseek(TO, $size - 1, 0);
+
+    my $buffer = pack('C', 0);
+
+    unless (syswrite(TO, $buffer))
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' =>
+		      ["cannot syswrite when creating sparse file <$file>"]);
+	return 0;
+    }
+
+    close(TO);
+    return 1;
+}
+
+
+############################################################
+# read pod2text
+sub getPod2Text
+{
+    my $file = shift;
+    my $prLog = shift;    # if not set, forkProc uses STDERR
+
+    my $p2t = forkProc->new('-exec' => 'pod2text',
+			    '-param' => [$file],
+			    '-outRandom' => "/tmp/pod2text-",
+			    '-prLog' => $prLog);
+    $p2t->wait();
+    my $out = $p2t->getSTDERR();
+    if (@$out > 0)
+    {
+	if ($prLog)
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' => ["STDERR of <pod2text>:", @$out])
+		if (@$out > 0);
+	}
+	else
+	{
+	    print STDERR "STDERR of <pod2text>:",
+	    join("\n", @$out), "\n";
+	}
+	exit 1;
+    }
+
+    $out = $p2t->getSTDOUT();
+
+    return join("\n", grep(!/^\s*$/, @$out)) . "\n";
+}
+
+
 ############################################################
 # Objekt kann zum (wiederholten) Abfragen von Informationen
 # ber eine Datei verwendet werden.
@@ -705,7 +789,8 @@ sub new
     my ($self) = {};
 
     my (%params) = ('-filename'      => undef,
-		    '-prLog'         => undef
+		    '-prLog'         => undef,
+		    '-tmpdir'        => '/tmp'
 		    );
 
     &::checkObjectParams(\%params, \@_, 'singleFileInfo::new',
@@ -738,6 +823,7 @@ sub getInfo
 			 ['-kind']);
     my $kind = $params{'-kind'};
     my $prLog = $self->{'prLog'};
+    my $tmpdir = $self->{'tmpdir'};
 
     if ($kind eq 'md5')
     {
@@ -747,8 +833,8 @@ sub getInfo
 	}
 
 	my $f = forkProc->new('-exec' => 'md5sum',
-			      '-stdout' => "/tmp/out.$$",
-			      '-stderr' => "/tmp/err.$$",
+			      '-stdout' => "$tmpdir/out.$$",
+			      '-stderr' => "$tmpdir/err.$$",
 			      '-param' => [$self->{'filename'}],
 			      '-prLog' => $prLog);
 	$f->wait();
@@ -810,6 +896,8 @@ sub new
 		    '-followLinks'         => 0, # nicht folgen, wenn 1, dann
 		                                 # in erster Ebene folgen, mehr
 		                                 # geht bisher nicht
+		    '-stayInFileSystem'    => 0, # don't leave file system, but
+		                                 # consider --followLinks
 		    '-prLog'               => undef,
 		    '-prLogError'          => 'E',
 		    '-prLogWarn'           => 'W',
@@ -855,6 +943,19 @@ sub new
     $self->{'depths'} = \@depths;
     $self->{'printedDepth'} = -1;
 
+    my %stayInFileSystemHash;
+    $self->{'stayInFileSystemHash'} = \%stayInFileSystemHash;
+    if ($self->{'stayInFileSystem'})
+    {
+	my $dirs = $self->{'dirs'};
+	foreach my $d (@$dirs)
+	{
+	    my $device = (stat($d))[0];    # device number of filesystem
+	    $stayInFileSystemHash{$device} = $d;
+#print "(1) $device --> $d\n";
+	}
+    }
+
     bless $self, $class;
 }
 
@@ -902,6 +1003,8 @@ sub readDir
     my $except = $self->{'except'};
     my $include = $self->{'include'};
     my $includeDirs = $self->{'includeDirs'};
+    my $stayInFileSystem = $self->{'stayInFileSystem'};
+    my $stayInFileSystemHash = $self->{'stayInFileSystemHash'};
     my $ignoreReadError = $self->{'ignoreReadError'};
 
 
@@ -983,18 +1086,26 @@ sub readDir
 	{
 	    $prLog->print('-kind' => $prLogErr,
 			  '-str' =>
-			  ["cannot opendir <$dir>, exiting",
-			   "check your source and clean your backup!"],
-			  '-exit' => $exit);
+			  ["cannot opendir <$dir>, " .
+			   "you may want to set option ignoreReadError to " .
+			   "make this message a warning"]);
+#			  '-exit' => $exit);
+	    return;
 	}
     }
 
     my $entry;
     my @notPlainFiles;
+#########!!!!!!!!
+##my $output = undef;
+##print "reading $dir\n";
     while ($entry = readdir DIR)
     {
 	next if ($entry eq '.' or $entry eq '..');
 	$entry = $dir . '/' . $entry;
+#########!!!!!!!!
+##print "$output\n" if $output;
+##$output = $entry;
 
 #	my $mode = (stat($entry))[2];
 #	if (::S_ISLNK($mode))
@@ -1003,15 +1114,24 @@ sub readDir
 	    if ($self->{'followLinks'} > $depth and
                -d "$entry/.")
 	    {
+#########!!!!!!!!
+##$output = "dir_a $output";
 		if (exists $$except{&::absolutePath($entry)})
 		{
 		    $prLog->print('-kind' => 'I',
-				  '-str' => ["ignoring directory <$entry>"])
+				  '-str' => ["ignoring directory <$entry>, " .
+				  "because of option exceptDir"])
 			if $self->{'verbose'};
 		    push @$files, $entry;
 		    push @$types, 'd';
 		    next;
 		}
+		if ($stayInFileSystem)
+		{
+		    my $device = (stat($entry))[0];
+		    $$stayInFileSystemHash{$device} = $entry;
+#print "(2) $device --> $entry\n";
+		}
 		push @$files, $entry;
 		push @$types, 'd';
 		push @$dirs, $entry;
@@ -1019,6 +1139,8 @@ sub readDir
 	    }
 	    else
 	    {
+#########!!!!!!!!
+##$output = "symlink $output";
 		push @$files, $entry;
 		push @$types, 'l';
 	    }
@@ -1027,6 +1149,9 @@ sub readDir
 	unless (-r $entry)
 #	unless ($mode & ::S_IREAD and $> != 0)  # $> -> effective user id
 	{
+	    next
+		if exists $$except{&::absolutePath($entry)};
+
 	    if ($ignoreReadError)
 	    {
 		$prLog->print('-kind' => $prLogWarn,
@@ -1035,14 +1160,34 @@ sub readDir
 	    else
 	    {
 		$prLog->print('-kind' => $prLogErr,
-			      '-str' => ["no permissions to read <$entry>"],
-			      '-exit' => 1);
+			      '-str' => ["no permissions to read <$entry>, " .
+			      "you may want to set option ignoreReadError to " .
+			      "make this message a warning"]);
+#			      '-exit' => 1);
 	    }
 	    next;
 	}
 	if (-d $entry)       # Dieses Directory mu beim Kopieren
 #	if (::S_ISDIR($mode))  # Dieses Directory mu beim Kopieren
 	{                      # z.B. mu angelegt werden!
+#########!!!!!!!!
+##$output = "dir $output";
+	    if ($stayInFileSystem)
+	    {
+		my $device = (stat($entry))[0];
+		if (not exists $$stayInFileSystemHash{$device})
+		{
+		    $prLog->print('-kind' => 'I',
+				  '-str' => ["ignoring directory <$entry>, " .
+				  "because of option stayInFileSystem"])
+			if $self->{'verbose'};
+		    push @$files, $entry;
+		    push @$types, 'd';
+#print "(3) nix $device --> $entry\n";
+		    next;
+		}
+#print "(2) $device --> $entry\n";
+	    }
 	    push @$files, $entry;
 	    push @$types, 'd';
 	    next if exists $$except{&::absolutePath($entry)};
@@ -1053,6 +1198,8 @@ sub readDir
 	if (-f $entry)
 #	if (::S_ISREG($mode))
 	{
+#########!!!!!!!!
+##$output = "file $output";
 	    push @$files, $entry;
 	    push @$types, 'f';
 	    next;
@@ -1060,6 +1207,8 @@ sub readDir
 	if (-p $entry)      # named pipe
 #	if (::S_ISFIFO($mode))      # named pipe
 	{
+#########!!!!!!!!
+##$output = "pipe $output";
 	    push @$files, $entry;
 	    push @$types, 'p';
 	    next;
@@ -1067,6 +1216,8 @@ sub readDir
 	if (-S $entry)      # socket
 #	if (::S_ISSOCK($mode))      # socket
 	{
+#########!!!!!!!!
+##$output = "socket $output";
 	    push @$files, $entry;
 	    push @$types, 'S';
 	    next;
@@ -1074,6 +1225,8 @@ sub readDir
 	if (-b $entry)      # block special file
 #	if (::S_ISBLK($mode))      # block special file
 	{
+#########!!!!!!!!
+##$output = "blockSpecial $output";
 	    push @$files, $entry;
 	    push @$types, 'b';
 	    next;
@@ -1081,6 +1234,8 @@ sub readDir
 	if (-c $entry)      # character special file
 #	if (::S_ISCHR($mode))      # character special file
 	{
+#########!!!!!!!!
+##$output = "charSpecial $output";
 	    push @$files, $entry;
 	    push @$types, 'c';
 	    next;
@@ -1089,6 +1244,9 @@ sub readDir
 		      '-str' => ["unsupported file type for <$entry>"]);
     }
 
+#########!!!!!!!!
+##print "$output\n" if $output;
+
     closedir DIR;
 }
 
