@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2001-2013)
+#   Copyright (C) Heinz-Josef Claes (2001-2014)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -66,7 +66,7 @@ sub waitForFile
 		      .05, .05, .05, .05, .05, .05, .05, .05,
 		     .05, .05, .05, .05, .05, .05, .05, .05, 0); # 1 second
 
-    if ((not -e $existingFile) and (stat($existingFile))[7] == 0)
+    if ((not -e $existingFile) or (stat($existingFile))[7] == 0)
     {
 	my $i;
 	foreach $i (@intervall)
@@ -83,7 +83,7 @@ sub waitForFile
 	if ($i == 0)
 	{
 	    system "/bin/sync";
-	    return (not -e $existingFile) and (stat($existingFile))[7] == 0;
+	    return (not -e $existingFile) or (stat($existingFile))[7] == 0;
 	}
     }
 
@@ -653,7 +653,7 @@ sub new
 		    '-delStdout'   => 'yes', # lsche Datei fr stdout
 		                             # automatisch
 
-		    '-prLog'       => undef,
+		    '-prLog'       => undef, # if not set, uses STDERR
 
 		    '-info'        => undef, # beliebige zustzliche Info
 
@@ -662,7 +662,7 @@ sub new
 		    );
 
     &::checkObjectParams(\%params, \@_, 'forkProc::new',
-			 ['-exec', '-prLog']);
+			 ['-exec']);
     &::setParamsDirect($self, \%params);
 
     my $proc = $params{'-exec'};
@@ -693,9 +693,16 @@ sub new
 	{
 	    unless (chdir "$dir")
 	    {
-		$prLog->print('-kind' => $err,
-			      '-str' =>
-			      ["Cannot chdir to <$dir> for <$proc>"]);
+		if ($prLog)
+		{
+		    $prLog->print('-kind' => $err,
+				  '-str' =>
+				  ["Cannot chdir to <$dir> for <$proc>"]);
+		}
+		else
+		{
+		    print STDERR "Cannot chdir to <$dir> for <$proc>\n";
+		}
 		exit $ex;
 	    }
 	}
@@ -703,9 +710,16 @@ sub new
 	{
 	    unless (sysopen(STDIN, $stdin, 0))
 	    {
-		$prLog->print('-kind' => $err,
-			      '-str' =>
-			      ["Cannot read <$stdin> when starting <$proc>"]);
+		if ($prLog)
+		{
+		    $prLog->print('-kind' => $err,
+				  '-str' =>
+				  ["Cannot read <$stdin> when starting <$proc>"]);
+		}
+		else
+		{
+		    print STDERR "Cannot read <$stdin> when starting <$proc>\n";
+		}
 		exit $ex;
 	    }
 	}
@@ -713,9 +727,16 @@ sub new
 	{
 	    unless (open STDOUT, "> $stdout")
 	    {
-		$prLog->print('-kind' => $err,
-			      '-str' =>
-			      ["Cannot write <$stdout> when starting <$proc>"]);
+		if ($prLog)
+		{
+		    $prLog->print('-kind' => $err,
+				  '-str' =>
+				  ["Cannot write <$stdout> when starting <$proc>"]);
+		}
+		else
+		{
+		    print STDERR "Cannot write <$stdout> when starting <$proc>\n";
+		}
 		exit $ex;
 	    }
 	    chown 0600, $stdout;
@@ -724,9 +745,16 @@ sub new
 	{
 	    unless (open STDERR, "> $stderr")
 	    {
-		$prLog->print('-kind' => $err,
-			      '-str' =>
-			      ["Cannot dup stderr when starting $proc"]);
+		if ($prLog)
+		{
+		    $prLog->print('-kind' => $err,
+				  '-str' =>
+				  ["Cannot dup stderr when starting $proc"]);
+		}
+		else
+		{
+		    print STDERR "Cannot dup stderr when starting $proc\n";
+		}
 		exit $ex;
 	    }
 	    chown 0600, $stderr;
@@ -734,8 +762,15 @@ sub new
 
 	unless (exec $proc, @$par)
 	{
-	    $prLog->print('-kind' => $err,
-			  '-str' => ["cannot exec $proc @$par"]);
+	    if ($prLog)
+	    {
+		$prLog->print('-kind' => $err,
+			      '-str' => ["cannot exec $proc @$par"]);
+	    }
+	    else
+	    {
+		print STDERR "cannot exec $proc @$par\n";
+	    }
 	    exit $ex;
 	}
     }
@@ -850,10 +885,18 @@ sub signal
 
     my $ret = kill $params{'-value'}, $self->{'pid'};
 
-    if ($ret) {
-	$self->{'prLog'}->print(-kind => 'W',
-				-str =>
-				["signaling process " . $self->{'pid'}]);
+    if ($ret)
+    {
+	if ($self->{'prLog'})
+	{
+	    $self->{'prLog'}->print(-kind => 'W',
+				    -str =>
+				    ["signaling process " . $self->{'pid'}]);
+	}
+	else
+	{
+	    print STDERR "signaling process " . $self->{'pid'} . "\n";
+	}
 	$self->delSTDOUT();
     }
     return $ret;
