@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2000-2012)
+#   Copyright (C) Heinz-Josef Claes (2000-2014)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -51,9 +51,10 @@ sub new
 		    '-multiprint'     => 'no',# pos an's Ende vor jedem print,
 		                              # mehrere Appl. print in ein log
 		    '-saveLogs'       => 'no',# nicht rundschreiben, wenn 'yes'
-		    '-compressWith'   => undef   # Komprimierungsprogr.,
+		    '-compressWith'   => undef,  # Komprimierungsprogr.,
 		                                 # z.B. 'gzip -9', 'bzip2'
 		                                 # nur zusammen mit '-saveLogs'
+		    '-tmpdir'         => '/tmp'  # used for forking (name pipe)
 		    );
 
     &::checkObjectParams(\%params, \@_, 'printLog::new', []);
@@ -126,14 +127,23 @@ sub new
 
 
 ##################################################
+# creates a named pipe and forks lib/stbuLog.pl
+#  which reads from that pipe
+# for each call of print (__reallyPrint) checks if 
+#  lib/stbuLog.pl was forked. If yes, writes to named
+#  pipe created
+#
+# to be used if program is forked (not execed!)
+# if fork exec, write to namend pipe in execed program directly
+#  get the name of the fifo with getFifoName (see below)
 sub fork
 {
     my $self = shift;
     my $pathTo_stbuLogPl = shift;
 
     return if defined $self->{'fifo'};
-
-    my $fifo = ::uniqFileName('/tmp/prLog-');
+    
+    my $fifo = ::uniqFileName($self->{'param'}{'-tmpdir'} . '/prLog-');
     POSIX::mkfifo($fifo, 0600) or
 	die "cannot mknod <$fifo> for printLog";
     $self->{'fifo'} = $fifo;
@@ -154,7 +164,7 @@ sub fork
 	}
 	else  # write round
 	{
-	    push @opts, ('--noOldFiles' => $self->{'param'}{'-noOfOldFiles'});
+	    push @opts, ('--noOfOldFiles' => $self->{'param'}{'-noOfOldFiles'});
 	}
     }
 
@@ -174,6 +184,15 @@ sub fork
 }
 
 
+##################################################
+sub getFifoName
+{
+    my $self = shift;
+
+    return $self->{'fifo'};
+}
+
+
 ##################################################
 sub setStopAtNoMessages
 {
@@ -374,7 +393,7 @@ sub __print         # interne Methode
 	    my (@t) = (localtime(time))[5,4,3,2,1,0];
 	    $t[0] += 1900;              # localtime liefert Zeit seit 1900
 	    $t[1]++;                    # Monat fngt bei 1 an
-	    my $t = sprintf("%04d.%02d.%02d_%02d:%02d:%02d", @t);
+	    my $t = sprintf("%04d.%02d.%02d_%02d.%02d.%02d", @t);
 	    my $t0 = $t;
 	    $t =~ s/\s/_/;         # blank zwischen Datum und Uhrzeit ersetzen
 	    $t =~ s/\s$//;         # blank am Ende lschen
@@ -505,7 +524,7 @@ sub __reallyPrint
 ##################################################
 sub DESTROY
 {
-    my ($self) = shift;
+    my $self = shift;
 
 #print "0\n";
 #print "fifo=",$self->{'fifo'},"\n";
@@ -682,3 +701,5 @@ sub pr
 	$prLog->pr(@_);
     }
 }
+
+1
