@@ -1,7 +1,7 @@
 #! /usr/bin/perl
 
 #
-#   Copyright (C) Heinz-Josef Claes (2009-2013)
+#   Copyright (C) Heinz-Josef Claes (2009-2014)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -50,7 +50,8 @@ my ($buffer, $n, $size);
 $size = 0;
 while ($n = sysread(IN, $buffer, 4096))
 {
-    syswrite OUT, $buffer;
+    print STDERR "cannot write to <$fileToSave>\n"
+	unless syswrite OUT, $buffer;
     $md5->add($buffer);
     $size += $n;
 }
